from __future__ import annotations

from datafusion._internal.common.data_type import DataTypeMap
from datafusion._internal.common.function import SqlFunction

class SqlSchema:
    def __init__(self, schema_name: str) -> None: ...
    def table_by_name(self, table_name: str) -> SqlTable | None: ...
    def add_table(self, table: SqlTable) -> None: ...
    def drop_table(self, table_name: str) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def tables(self) -> list[SqlTable]: ...
    @property
    def views(self) -> list[SqlView]: ...
    @property
    def functions(self) -> list[SqlFunction]: ...

class SqlTable:
    def __init__(
        self,
        table_name: str,
        columns: list[tuple[str, DataTypeMap]],
        row_count: float,
        filepaths: list[str] | None,
    ) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def columns(self) -> list[tuple[str, DataTypeMap]]: ...
    @property
    def primary_key(self) -> str | None: ...
    @property
    def foreign_keys(self) -> list[str]: ...
    @property
    def constraints(self) -> list[str]: ...
    @property
    def statistics(self) -> SqlStatistics: ...
    @property
    def filepaths(self) -> list[str] | None: ...

class SqlView:
    @property
    def name(self) -> str: ...
    @property
    def definition(self) -> str: ...

class SqlStatistics:
    def __init__(self, row_count: float) -> None: ...
    def get_row_count(self) -> float: ...
    @property
    def row_count(self) -> float: ...
