from __future__ import annotations

from typing import TYPE_CHECKING

from datafusion._internal.context import SessionContext
from datafusion._internal.expr import Expr
from datafusion._internal.expr.conditional_expr import CaseBuilder
from datafusion._internal.expr.window import WindowFrame

def isnan(expr: Expr) -> Expr: ...
def nullif(expr1: Expr, expr2: Expr) -> Expr: ...
def encode(input1: Expr, encoding: Expr) -> Expr: ...
def decode(input1: Expr, encoding: Expr) -> Expr: ...
def array_to_string(expr: Expr, delim: Expr) -> Expr: ...
def array_join(expr: Expr, delim: Expr) -> Expr: ...
def list_to_string(expr: Expr, delim: Expr) -> Expr: ...
def list_join(expr: Expr, delim: Expr) -> Expr: ...
def in_list(expr: Expr, value: list[Expr], negated: bool) -> Expr: ...
def digest(value: Expr, method: Expr) -> Expr: ...
def concat(*args: Expr) -> Expr: ...
def concat_ws(sep: str, *args: Expr) -> Expr: ...
def order_by(expr: Expr, asc: bool | None, nulls_first: bool | None) -> Expr: ...
def alias(expr: Expr, name: str) -> Expr: ...
def col(name: str) -> Expr: ...
def count_star() -> Expr: ...
def case(expr: Expr) -> CaseBuilder: ...
def window(
    name: str,
    args: list[Expr],
    partition_by: list[Expr] | None,
    order_by: list[Expr] | None,
    window_frame: WindowFrame | None,
    ctx: SessionContext,
) -> Expr: ...

# scalar functions
def abs(*args: list[Expr]) -> Expr: ...
def acos(*args: list[Expr]) -> Expr: ...
def acosh(*args: list[Expr]) -> Expr: ...
def ascii(*args: list[Expr]) -> Expr: ...
def asin(*args: list[Expr]) -> Expr: ...
def asinh(*args: list[Expr]) -> Expr: ...
def atan(*args: list[Expr]) -> Expr: ...
def atanh(*args: list[Expr]) -> Expr: ...
def atan2(*args: list[Expr]) -> Expr: ...
def bit_length(*args: list[Expr]) -> Expr: ...
def btrim(*args: list[Expr]) -> Expr: ...
def cbrt(*args: list[Expr]) -> Expr: ...
def ceil(*args: list[Expr]) -> Expr: ...
def character_length(*args: list[Expr]) -> Expr: ...
def length(*args: list[Expr]) -> Expr: ...
def char_length(*args: list[Expr]) -> Expr: ...
def chr(*args: list[Expr]) -> Expr: ...
def coalesce(*args: list[Expr]) -> Expr: ...
def cos(*args: list[Expr]) -> Expr: ...
def cosh(*args: list[Expr]) -> Expr: ...
def degrees(*args: list[Expr]) -> Expr: ...
def exp(*args: list[Expr]) -> Expr: ...
def factorial(*args: list[Expr]) -> Expr: ...
def floor(*args: list[Expr]) -> Expr: ...
def gcd(*args: list[Expr]) -> Expr: ...
def initcap(*args: list[Expr]) -> Expr: ...
def iszero(*args: list[Expr]) -> Expr: ...
def lcm(*args: list[Expr]) -> Expr: ...
def left(*args: list[Expr]) -> Expr: ...
def ln(*args: list[Expr]) -> Expr: ...
def log(*args: list[Expr]) -> Expr: ...
def log10(*args: list[Expr]) -> Expr: ...
def log2(*args: list[Expr]) -> Expr: ...
def lower(*args: list[Expr]) -> Expr: ...
def lpad(*args: list[Expr]) -> Expr: ...
def ltrim(*args: list[Expr]) -> Expr: ...
def md5(*args: list[Expr]) -> Expr: ...
def nanvl(*args: list[Expr]) -> Expr: ...
def octet_length(*args: list[Expr]) -> Expr: ...
def pi(*args: list[Expr]) -> Expr: ...
def power(*args: list[Expr]) -> Expr: ...
def pow(*args: list[Expr]) -> Expr: ...
def radians(*args: list[Expr]) -> Expr: ...
def regexp_match(*args: list[Expr]) -> Expr: ...
def regexp_replace(*args: list[Expr]) -> Expr: ...
def repeat(*args: list[Expr]) -> Expr: ...
def replace(*args: list[Expr]) -> Expr: ...
def reverse(*args: list[Expr]) -> Expr: ...
def right(*args: list[Expr]) -> Expr: ...
def round(*args: list[Expr]) -> Expr: ...
def rpad(*args: list[Expr]) -> Expr: ...
def rtrim(*args: list[Expr]) -> Expr: ...
def sha224(*args: list[Expr]) -> Expr: ...
def sha256(*args: list[Expr]) -> Expr: ...
def sha384(*args: list[Expr]) -> Expr: ...
def sha512(*args: list[Expr]) -> Expr: ...
def signum(*args: list[Expr]) -> Expr: ...
def sin(*args: list[Expr]) -> Expr: ...
def sinh(*args: list[Expr]) -> Expr: ...
def split_part(*args: list[Expr]) -> Expr: ...
def sqrt(*args: list[Expr]) -> Expr: ...
def starts_with(*args: list[Expr]) -> Expr: ...
def strpos(*args: list[Expr]) -> Expr: ...
def substr(*args: list[Expr]) -> Expr: ...
def tan(*args: list[Expr]) -> Expr: ...
def tanh(*args: list[Expr]) -> Expr: ...
def to_hex(*args: list[Expr]) -> Expr: ...
def now(*args: list[Expr]) -> Expr: ...
def to_timestamp(*args: list[Expr]) -> Expr: ...
def to_timestamp_millis(*args: list[Expr]) -> Expr: ...
def to_timestamp_micros(*args: list[Expr]) -> Expr: ...
def to_timestamp_seconds(*args: list[Expr]) -> Expr: ...
def current_date(*args: list[Expr]) -> Expr: ...
def current_time(*args: list[Expr]) -> Expr: ...
def datepart(*args: list[Expr]) -> Expr: ...
def date_part(*args: list[Expr]) -> Expr: ...
def date_trunc(*args: list[Expr]) -> Expr: ...
def datetrunc(*args: list[Expr]) -> Expr: ...
def date_bin(*args: list[Expr]) -> Expr: ...
def translate(*args: list[Expr]) -> Expr: ...
def trim(*args: list[Expr]) -> Expr: ...
def trunc(*args: list[Expr]) -> Expr: ...
def upper(*args: list[Expr]) -> Expr: ...
def make_array(*args: list[Expr]) -> Expr: ...
def array(*args: list[Expr]) -> Expr: ...
def range(*args: list[Expr]) -> Expr: ...
def uuid(*args: list[Expr]) -> Expr: ...
def struct(*args: list[Expr]) -> Expr: ...
def from_unixtime(*args: list[Expr]) -> Expr: ...
def arrow_typeof(*args: list[Expr]) -> Expr: ...
def random(*args: list[Expr]) -> Expr: ...
def array_append(*args: list[Expr]) -> Expr: ...
def array_push_back(*args: list[Expr]) -> Expr: ...
def list_append(*args: list[Expr]) -> Expr: ...
def list_push_back(*args: list[Expr]) -> Expr: ...
def array_concat(*args: list[Expr]) -> Expr: ...
def array_cat(*args: list[Expr]) -> Expr: ...
def array_dims(*args: list[Expr]) -> Expr: ...
def array_distinct(*args: list[Expr]) -> Expr: ...
def list_distinct(*args: list[Expr]) -> Expr: ...
def list_dims(*args: list[Expr]) -> Expr: ...
def array_element(*args: list[Expr]) -> Expr: ...
def array_extract(*args: list[Expr]) -> Expr: ...
def list_element(*args: list[Expr]) -> Expr: ...
def list_extract(*args: list[Expr]) -> Expr: ...
def array_length(*args: list[Expr]) -> Expr: ...
def list_length(*args: list[Expr]) -> Expr: ...
def array_has(*args: list[Expr]) -> Expr: ...
def array_has_all(*args: list[Expr]) -> Expr: ...
def array_has_any(*args: list[Expr]) -> Expr: ...
def array_position(*args: list[Expr]) -> Expr: ...
def array_indexof(*args: list[Expr]) -> Expr: ...
def list_position(*args: list[Expr]) -> Expr: ...
def list_indexof(*args: list[Expr]) -> Expr: ...
def array_positions(*args: list[Expr]) -> Expr: ...
def list_positions(*args: list[Expr]) -> Expr: ...
def array_ndims(*args: list[Expr]) -> Expr: ...
def list_ndims(*args: list[Expr]) -> Expr: ...
def array_prepend(*args: list[Expr]) -> Expr: ...
def array_push_front(*args: list[Expr]) -> Expr: ...
def list_prepend(*args: list[Expr]) -> Expr: ...
def list_push_front(*args: list[Expr]) -> Expr: ...
def array_pop_back(*args: list[Expr]) -> Expr: ...
def array_pop_front(*args: list[Expr]) -> Expr: ...
def array_remove(*args: list[Expr]) -> Expr: ...
def list_remove(*args: list[Expr]) -> Expr: ...
def array_remove_n(*args: list[Expr]) -> Expr: ...
def list_remove_n(*args: list[Expr]) -> Expr: ...
def array_remove_all(*args: list[Expr]) -> Expr: ...
def list_remove_all(*args: list[Expr]) -> Expr: ...
def array_repeat(*args: list[Expr]) -> Expr: ...
def array_replace(*args: list[Expr]) -> Expr: ...
def list_replace(*args: list[Expr]) -> Expr: ...
def array_replace_n(*args: list[Expr]) -> Expr: ...
def list_replace_n(*args: list[Expr]) -> Expr: ...
def array_replace_all(*args: list[Expr]) -> Expr: ...
def list_replace_all(*args: list[Expr]) -> Expr: ...
def array_slice(*args: list[Expr]) -> Expr: ...
def list_slice(*args: list[Expr]) -> Expr: ...
def array_intersect(*args: list[Expr]) -> Expr: ...
def list_intersect(*args: list[Expr]) -> Expr: ...
def array_union(*args: list[Expr]) -> Expr: ...
def list_union(*args: list[Expr]) -> Expr: ...
def array_except(*args: list[Expr]) -> Expr: ...
def list_except(*args: list[Expr]) -> Expr: ...
def array_resize(*args: list[Expr]) -> Expr: ...
def list_resize(*args: list[Expr]) -> Expr: ...
def flatten(*args: list[Expr]) -> Expr: ...

# aggregate functions
def approx_distinct(*args: list[Expr], distinct: bool = False) -> Expr: ...
def approx_median(*args: list[Expr], distinct: bool = False) -> Expr: ...
def approx_percentile_cont(*args: list[Expr], distinct: bool = False) -> Expr: ...
def approx_percentile_cont_with_weight(
    *args: list[Expr], distinct: bool = False
) -> Expr: ...
def array_agg(*args: list[Expr], distinct: bool = False) -> Expr: ...
def avg(*args: list[Expr], distinct: bool = False) -> Expr: ...
def corr(*args: list[Expr], distinct: bool = False) -> Expr: ...
def count(*args: list[Expr], distinct: bool = False) -> Expr: ...
def covar(*args: list[Expr], distinct: bool = False) -> Expr: ...
def covar_pop(*args: list[Expr], distinct: bool = False) -> Expr: ...
def covar_samp(*args: list[Expr], distinct: bool = False) -> Expr: ...
def grouping(*args: list[Expr], distinct: bool = False) -> Expr: ...
def max(*args: list[Expr], distinct: bool = False) -> Expr: ...
def mean(*args: list[Expr], distinct: bool = False) -> Expr: ...
def median(*args: list[Expr], distinct: bool = False) -> Expr: ...
def min(*args: list[Expr], distinct: bool = False) -> Expr: ...
def sum(*args: list[Expr], distinct: bool = False) -> Expr: ...
def stddev(*args: list[Expr], distinct: bool = False) -> Expr: ...
def stddev_pop(*args: list[Expr], distinct: bool = False) -> Expr: ...
def stddev_samp(*args: list[Expr], distinct: bool = False) -> Expr: ...
def var(*args: list[Expr], distinct: bool = False) -> Expr: ...
def var_pop(*args: list[Expr], distinct: bool = False) -> Expr: ...
def var_samp(*args: list[Expr], distinct: bool = False) -> Expr: ...
def regr_avgx(*args: list[Expr], distinct: bool = False) -> Expr: ...
def regr_avgy(*args: list[Expr], distinct: bool = False) -> Expr: ...
def regr_count(*args: list[Expr], distinct: bool = False) -> Expr: ...
def regr_intercept(*args: list[Expr], distinct: bool = False) -> Expr: ...
def regr_r2(*args: list[Expr], distinct: bool = False) -> Expr: ...
def regr_slope(*args: list[Expr], distinct: bool = False) -> Expr: ...
def regr_sxx(*args: list[Expr], distinct: bool = False) -> Expr: ...
def regr_sxy(*args: list[Expr], distinct: bool = False) -> Expr: ...
def regr_syy(*args: list[Expr], distinct: bool = False) -> Expr: ...
def first_value(*args: list[Expr], distinct: bool = False) -> Expr: ...
def last_value(*args: list[Expr], distinct: bool = False) -> Expr: ...
def bit_and(*args: list[Expr], distinct: bool = False) -> Expr: ...
def bit_or(*args: list[Expr], distinct: bool = False) -> Expr: ...
def bit_xor(*args: list[Expr], distinct: bool = False) -> Expr: ...
def bool_and(*args: list[Expr], distinct: bool = False) -> Expr: ...
def bool_or(*args: list[Expr], distinct: bool = False) -> Expr: ...
